@@ -18,7 +18,7 @@ import pyubx2.ubxtypes_core as ubt
 
 from .globals import CONNECTED, CONNECTED_FILE, DISCONNECTED, SERIAL_TIMEOUT, \
                                 NMEA_PROTOCOL, MIXED_PROTOCOL, UBX_PROTOCOL, PARITIES
-from .strings import WAITUBXDATA, STOPDATA, NOTCONN, SEROPENERROR
+from .strings import STOPDATA, NOTCONN, SEROPENERROR, ENDOFFILE
 
 
 class SerialHandler():
@@ -37,6 +37,7 @@ class SerialHandler():
         self._serial_object = None
         self._serial_buffer = None
         self._serial_thread = None
+        self._file_thread = None
         self._connected = False
         self._logpath = None
         self._datalogging = False
@@ -104,10 +105,10 @@ class SerialHandler():
             self._serial_object = open(self._logpath, 'rb')
             self._serial_buffer = BufferedReader(self._serial_object)
             self.__app.frm_banner.update_banner(status=CONNECTED_FILE)
-            self.__app.set_connection(f"{self._logpath}", "green")
+            self.__app.set_connection(f"{self._logpath}", "blue")
             self.__app.frm_settings.set_controls(CONNECTED_FILE)
             self._connected = True
-            self.start_read_thread()
+            self.start_readfile_thread()
         except (IOError, SerialException, SerialTimeoutException) as err:
             self._connected = False
             self.__app.set_connection(f"{self._logpath}", "red")
@@ -180,7 +181,6 @@ class SerialHandler():
 
         try:
             self._serial_object.write(data)
-#             self._serial_buffer.write(data)
         except (SerialException, SerialTimeoutException) as err:
             print(f"Error writing to serial port {err}")
 
@@ -191,13 +191,21 @@ class SerialHandler():
 
         if self._connected:
             self._reading = True
-            self.__app.set_status(WAITUBXDATA, "blue")
             self.__app.frm_mapview.reset_map_refresh()
-            if self._serial_object.isatty():
-                self._serial_object.reset_input_buffer()
             self._serial_thread = Thread(target=self._read_thread, daemon=True)
             self._serial_thread.start()
 
+    def start_readfile_thread(self):
+        '''
+        Start the file reader thread.
+        '''
+
+        if self._connected:
+            self._reading = True
+            self.__app.frm_mapview.reset_map_refresh()
+            self._file_thread = Thread(target=self._readfile_thread, daemon=True)
+            self._file_thread.start()
+
     def stop_read_thread(self):
         '''
         Stop serial reader thread.
@@ -208,6 +216,16 @@ class SerialHandler():
             self._serial_thread = None
             self.__app.set_status(STOPDATA, "red")
 
+    def stop_readfile_thread(self):
+        '''
+        Stop file reader thread.
+        '''
+
+        if self._file_thread is not None:
+            self._reading = False
+            self._file_thread = None
+            self.__app.set_status(STOPDATA, "red")
+
     def _read_thread(self):
         '''
         THREADED PROCESS
@@ -215,46 +233,81 @@ class SerialHandler():
         trigger data parsing and widget updates.
         '''
 
+#         print("doing serial_handler._read_thread")
         while self._reading and self._serial_object:
-            # print(f"Bytes in buffer: {self._serial_object.in_waiting}")
-            if self._serial_object.isatty():
-                if self._serial_object.in_waiting:
-                    self.__master.event_generate('<<ubx_read>>')
-            else:
-                self.__master.event_generate('<<ubx_readfile>>')
+#             print("doing serial_handler._read_thread while loop")
+            if self._serial_object.in_waiting:
+#                 print(f"Bytes in buffer: {self._serial_object.in_waiting}")
+#                 print("doing serial_handler._read_thread in_waiting")
+                self.__master.event_generate('<<ubx_read>>')
+
+    def _readfile_thread(self):
+        '''
+        THREADED PROCESS
+        Reads binary data from datalog file and generates virtual event to
+        trigger data parsing and widget updates.
+        '''
+
+#         print("doing serial_handler._readfile_thread")
+        while self._reading and self._serial_object:
+#             print("doing serial_handler._readfile_thread while loop")
+            self.__master.event_generate('<<ubx_readfile>>')
 
     def on_read(self, event):  # pylint: disable=unused-argument
         '''
         Action on <<ubx_read>> event - read any data in the buffer.
         '''
 
+#         print("doing serial_handler.on_read")
         if self._reading and self._serial_object is not None:
-            self.__app.set_status("",)
-            self._parse_data(self._serial_buffer)
+            try:
+                self._parse_data(self._serial_buffer)
+            except SerialException as err:
+                self.__app.set_status(f"Error {err}", "red")
 
-    def _parse_data(self, ser: Serial) -> object:
+    def on_eof(self, event):  # pylint: disable=unused-argument
+        '''
+        Action on end of file
+        '''
+
+#         print("doing serial_handler.on_eof")
+        self.disconnect()
+        self.__app.set_status(ENDOFFILE, "blue")
+
+    def _parse_data(self, ser: Serial):
         '''
         Read the binary data and direct to the appropriate
         UBX and/or NMEA protocol handler, depending on which protocols
         are filtered.
         '''
 
+#         print("doing serial_handler_parse_data")
         parsing = True
         raw_data = None
         byte1 = ser.read(2)  # read first two bytes to determine protocol
+        if len(byte1) < 2:
+            self.__master.event_generate('<<ubx_eof>>')
+            return
 
         while parsing:
-#             self.__app.update_idletasks()
             filt = self.__app.frm_settings.get_settings()['protocol']
             # if it's a UBX message (b'\b5\x62')
             if byte1 == ubt.UBX_HDR and filt in (UBX_PROTOCOL, MIXED_PROTOCOL):
-#                 print(f"doing ubx {ser.peek()}")
+#                 print(f"doing serial_handler._parse_data if ubx {ser.peek()}")
                 byten = ser.read(4)
+                if len(byten) < 4:
+                    self.__master.event_generate('<<ubx_eof>>')
+                    parsing = False
+                    break
                 clsid = byten[0:1]
                 msgid = byten[1:2]
                 lenb = byten[2:4]
                 leni = int.from_bytes(lenb, 'little', signed=False)
                 byten = ser.read(leni + 2)
+                if len(byten) < leni + 2:
+                    self.__master.event_generate('<<ubx_eof>>')
+                    parsing = False
+                    break
                 plb = byten[0:leni]
                 cksum = byten[leni:leni + 2]
                 raw_data = ubt.UBX_HDR + clsid + msgid + lenb + plb + cksum
@@ -262,7 +315,7 @@ class SerialHandler():
                 parsing = False
             # if it's an NMEA message ('$G' or '$P')
             elif byte1 in (b'\x24\x47', b'\x24\x50') and filt in (NMEA_PROTOCOL, MIXED_PROTOCOL):
-#                 print(f"doing nmea {ser.peek()}")
+#                 print(f"doing serial_handler._parse_data if nmea {ser.peek()}")
                 try:
                     raw_data = byte1 + ser.readline()
                     self.__app.nmea_handler.process_data(raw_data.decode("utf-8"))
