@@ -10,29 +10,29 @@ Created on 22 Sep 2020
 '''
 # pylint: disable=invalid-name, unused-argument
 
-from tkinter import ttk, messagebox, Toplevel, Frame, Checkbutton, Radiobutton, Listbox, \
+from tkinter import ttk, messagebox, Toplevel, Frame, Radiobutton, Listbox, \
                     Spinbox, Scrollbar, Button, Label, StringVar, \
                     IntVar, N, S, E, W, LEFT, VERTICAL, HORIZONTAL
 
 from PIL import ImageTk, Image
-from pyubx2 import UBXMessage, POLL, SET, UBX_CONFIG_MESSAGES
+from pyubx2 import UBXMessage, POLL, SET, UBX_CONFIG_MESSAGES, UBX_PAYLOADS_POLL
 
 from .globals import BGCOL, FGCOL, ENTCOL, ICON_APP, ICON_SEND, ICON_EXIT, ICON_WARNING, \
-                     ICON_PENDING, ICON_CONFIRMED, BAUDRATES, READONLY
+                     ICON_UBXCONFIG, ICON_PENDING, ICON_CONFIRMED, BAUDRATES, READONLY
 from .strings import LBLUBXCONFIG, LBLCFGPRT, LBLCFGMSG, LBLPRESET, DLGUBXCONFIG, DLGRESET, \
                      DLGSAVE, DLGRESETCONFIRM, DLGSAVECONFIRM, PSTRESET, PSTSAVE, \
                      PSTMINNMEAON, PSTALLNMEAON, PSTALLNMEAOFF, PSTMINUBXON, PSTALLUBXON, \
                      PSTALLUBXOFF, PSTALLINFON, PSTALLINFOFF, PSTALLLOGON, PSTALLLOGOFF, \
                      PSTALLMONON, PSTALLMONOFF, PSTALLRXMON, PSTALLRXMOFF, PSTPOLLPORT, \
-                     PSTPOLLINFO
+                     PSTPOLLINFO, PSTPOLLALL
 
-from .ubx_handler import UBXHandler as ubh, CFG_MSG_OFF, CFG_MSG_ON
+from .ubx_handler import UBXHandler as ubh
 
 PRESET_COMMMANDS = [
 PSTRESET, PSTSAVE, PSTMINNMEAON, PSTALLNMEAON, PSTALLNMEAOFF,
 PSTMINUBXON, PSTALLUBXON, PSTALLUBXOFF, PSTALLINFON, PSTALLINFOFF,
 PSTALLLOGON, PSTALLLOGOFF, PSTALLMONON, PSTALLMONOFF, PSTALLRXMON,
-PSTALLRXMOFF, PSTPOLLPORT, PSTPOLLINFO
+PSTALLRXMOFF, PSTPOLLPORT, PSTPOLLINFO, PSTPOLLALL
 ]
 
 
@@ -56,7 +56,7 @@ class UBXConfigDialog():
         wa = self.__master.winfo_width()
         ha = self.__master.winfo_height()
         self._dialog.geometry("+%d+%d" % (int(wa / 2 - wd / 2), int(ha / 2 - hd / 2)))
-        self._img_icon = ImageTk.PhotoImage(Image.open (ICON_APP))
+        self._img_icon = ImageTk.PhotoImage(Image.open (ICON_UBXCONFIG))
         self._img_pending = ImageTk.PhotoImage(Image.open(ICON_PENDING))
         self._img_confirmed = ImageTk.PhotoImage(Image.open(ICON_CONFIRMED))
         self._img_send = ImageTk.PhotoImage(Image.open(ICON_SEND))
@@ -73,10 +73,15 @@ class UBXConfigDialog():
         self._ubx_baudrate = StringVar()
         self._ubx_inprot = IntVar()
         self._ubx_outprot = IntVar()
-        self._ddc_onoff = IntVar()
-        self._uart1_onoff = IntVar()
-        self._usb_onoff = IntVar()
-        self._spi_onoff = IntVar()
+        self._ddc_rate = IntVar()
+        self._uart1_rate = IntVar()
+        self._usb_rate = IntVar()
+        self._spi_rate = IntVar()
+        self._sw_version = StringVar()
+        self._hw_version = StringVar()
+        self._fw_version = StringVar()
+        self._protocol = StringVar()
+        self._gnss_supported = StringVar()
 
         self._body()
         self._do_layout()
@@ -98,7 +103,25 @@ class UBXConfigDialog():
         con.option_add("*Font", self.__app.font_sm)
 
         self._lbl_title = Label(con, text=LBLUBXCONFIG, bg=BGCOL, fg=FGCOL,
-                                justify=LEFT, font=self.__app.font_md, width=55)
+                                justify=LEFT, font=self.__app.font_md)
+
+        # *******************************************************
+        # Software and Hardware versions
+        # *******************************************************
+        INFCOL = 'steelblue3'
+        self._frm_device_info = Frame(con, borderwidth=2)
+        inf = self._frm_device_info
+        self._lbl_swverl = Label(inf, text="Software")
+        self._lbl_swver = Label(inf, textvariable=self._sw_version, fg=INFCOL)
+        self._lbl_hwverl = Label(inf, text="Hardware")
+        self._lbl_hwver = Label(inf, textvariable=self._hw_version, fg=INFCOL)
+        self._lbl_fwverl = Label(inf, text="Firmware")
+        self._lbl_fwver = Label(inf, textvariable=self._fw_version, fg=INFCOL)
+        self._lbl_romverl = Label(inf, text="Protocol")
+        self._lbl_romver = Label(inf, textvariable=self._protocol, fg=INFCOL)
+        self._lbl_gnssl = Label(inf, text="GNSS/AS")
+        self._lbl_gnss = Label(inf, textvariable=self._gnss_supported, fg=INFCOL)
+
         # *******************************************************
         # Port Configuration
         # *******************************************************
@@ -125,7 +148,7 @@ class UBXConfigDialog():
                                             variable=self._ubx_outprot, value=1)
         self._rad_outprot_both = Radiobutton(con, text="ALL",
                                              variable=self._ubx_outprot, value=3)
-        self._lbl_send_port = Label(con)
+        self._lbl_send_port = Label(con, image=self._img_pending)
         self._btn_send_port = Button(con, image=self._img_send, width=50,
                                      command=self._on_send_port,
                                      font=self.__app.font_md)
@@ -133,19 +156,32 @@ class UBXConfigDialog():
         # *******************************************************
         # CFG-MSG Message Selection
         # *******************************************************
+        MAX_RATE = 0xff
         self._lbl_cfg_msg = Label(con, text=LBLCFGMSG,
                                   bg=LBL_COL, anchor='w')
         self._lbx_cfg_msg = Listbox(con, border=2,
                                  relief="sunken", bg=ENTCOL,
-                                 height=7, justify=LEFT,
+                                 height=6, justify=LEFT,
                                  exportselection=False)
         self._scr_cfg_msg = Scrollbar(con, orient=VERTICAL)
         self._lbx_cfg_msg.config(yscrollcommand=self._scr_cfg_msg.set)
         self._scr_cfg_msg.config(command=self._lbx_cfg_msg.yview)
-        self._chk_ddc = Checkbutton(con, text='DDC (I2C)', variable=self._ddc_onoff)
-        self._chk_uart1 = Checkbutton(con, text='UART1', variable=self._uart1_onoff)
-        self._chk_usb = Checkbutton(con, text='USB', variable=self._usb_onoff)
-        self._chk_spi = Checkbutton(con, text='SPI', variable=self._spi_onoff)
+        self._lbl_ddc = Label(con, text='DDC (I2C)')
+        self._spn_ddc = Spinbox(con, width=3, from_=0, to=MAX_RATE,
+                                textvariable=self._ddc_rate,
+                                state=READONLY, readonlybackground=ENTCOL)
+        self._lbl_uart1 = Label(con, text='UART1')
+        self._spn_uart1 = Spinbox(con, width=3, from_=0, to=MAX_RATE,
+                                  textvariable=self._uart1_rate,
+                                  state=READONLY, readonlybackground=ENTCOL)
+        self._lbl_usb = Label(con, text='USB')
+        self._spn_usb = Spinbox(con, width=3, from_=0, to=MAX_RATE,
+                                textvariable=self._usb_rate,
+                                state=READONLY, readonlybackground=ENTCOL)
+        self._lbl_spi = Label(con, text='SPI')
+        self._spn_spi = Spinbox(con, width=3, from_=0, to=MAX_RATE,
+                                textvariable=self._spi_rate,
+                                state=READONLY, readonlybackground=ENTCOL)
         self._lbl_send_cfg_msg = Label(con)
         self._btn_send_cfg_msg = Button(con, image=self._img_send, width=50, fg="green",
                                        command=self._on_send_cfg_msg, font=self.__app.font_md)
@@ -158,7 +194,7 @@ class UBXConfigDialog():
                                   bg=LBL_COL, anchor='w')
         self._lbx_preset = Listbox(con, border=2,
                                  relief="sunken", bg=ENTCOL,
-                                 height=7, justify=LEFT,
+                                 height=5, justify=LEFT,
                                  exportselection=False)
         self._scr_presetv = Scrollbar(con, orient=VERTICAL)
         self._scr_preseth = Scrollbar(con, orient=HORIZONTAL)
@@ -182,61 +218,86 @@ class UBXConfigDialog():
         self._frm_container.grid(column=0, row=0, columnspan=6, padx=3, pady=3,
                                  ipadx=5, ipady=5, sticky=(N, S, W, E))
         self._lbl_title.grid(column=0, row=0, columnspan=6, ipady=3, sticky=(W, E))
-        self._lbl_cfg_port.grid(column=0, row=1, columnspan=6, padx=3, sticky=(W, E))
-        self._lbl_ubx_baudrate.grid(column=0, row=2, columnspan=3, sticky=(W))
-        self._spn_ubx_baudrate.grid(column=1, row=2, columnspan=3, sticky=(W))
-        self._lbl_inprot.grid(column=0, row=3, sticky=(W))
-        self._rad_inprot_nmea.grid(column=1, row=3, sticky=(W))
-        self._rad_inprot_ubx.grid(column=2, row=3, sticky=(W))
-        self._rad_inprot_rtcm.grid(column=3, row=3, sticky=(W))
-        self._rad_inprot_both.grid(column=4, row=3, sticky=(W))
-        self._lbl_outprot.grid(column=0, row=4, sticky=(W))
-        self._rad_outprot_nmea.grid(column=1, row=4, sticky=(W))
-        self._rad_outprot_ubx.grid(column=2, row=4, sticky=(W))
-        self._rad_outprot_both.grid(column=3, row=4, sticky=(W))
-        self._btn_send_port.grid(column=4, row=2, ipadx=3,
+
+        # *******************************************************
+        # MON-VER Device Information
+        # *******************************************************
+        self._frm_device_info.grid(column=0, row=1, columnspan=6, sticky=(W, E))
+        self._lbl_swverl.grid(column=0, row=0, padx=2, sticky=(W))
+        self._lbl_swver.grid(column=1, row=0, columnspan=2, padx=2, sticky=(W))
+        self._lbl_hwverl.grid(column=3, row=0, padx=2, sticky=(W))
+        self._lbl_hwver.grid(column=4, row=0, columnspan=2, padx=2, sticky=(W))
+        self._lbl_fwverl.grid(column=0, row=1, padx=2, sticky=(W))
+        self._lbl_fwver.grid(column=1, row=1, columnspan=2, padx=2, sticky=(W))
+        self._lbl_romverl.grid(column=3, row=1, padx=2, sticky=(W))
+        self._lbl_romver.grid(column=4, row=1, columnspan=2, padx=2, sticky=(W))
+        self._lbl_gnssl.grid(column=0, row=2, columnspan=2, padx=2, sticky=(W))
+        self._lbl_gnss.grid(column=2, row=2, columnspan=4, padx=2, sticky=(W))
+
+        # *******************************************************
+        # CFG-PRT Port Configuration
+        # *******************************************************
+        ttk.Separator(self._frm_container).grid(column=0, row=2, columnspan=6,
+                                                padx=3, pady=3, sticky=(W, E))
+        self._lbl_cfg_port.grid(column=0, row=3, columnspan=6, padx=3, sticky=(W, E))
+        self._lbl_ubx_baudrate.grid(column=0, row=4, columnspan=3, sticky=(W))
+        self._spn_ubx_baudrate.grid(column=1, row=4, columnspan=3, sticky=(W))
+        self._lbl_inprot.grid(column=0, row=5, sticky=(W))
+        self._rad_inprot_nmea.grid(column=1, row=5, sticky=(W))
+        self._rad_inprot_ubx.grid(column=2, row=5, sticky=(W))
+        self._rad_inprot_rtcm.grid(column=3, row=5, sticky=(W))
+        self._rad_inprot_both.grid(column=4, row=5, sticky=(W))
+        self._lbl_outprot.grid(column=0, row=6, sticky=(W))
+        self._rad_outprot_nmea.grid(column=1, row=6, sticky=(W))
+        self._rad_outprot_ubx.grid(column=2, row=6, sticky=(W))
+        self._rad_outprot_both.grid(column=3, row=6, sticky=(W))
+        self._btn_send_port.grid(column=4, row=4, ipadx=3,
                                  ipady=3, sticky=(E))
-        self._lbl_send_port.grid(column=5, row=2, ipadx=3,
+        self._lbl_send_port.grid(column=5, row=4, ipadx=3,
                                  ipady=3, sticky=(W))
 
         # *******************************************************
         # CFG-MSG Message Selection
         # *******************************************************
-        ttk.Separator(self._frm_container).grid(column=0, row=5, columnspan=6,
+        ttk.Separator(self._frm_container).grid(column=0, row=7, columnspan=6,
                                                 padx=3, pady=3, sticky=(W, E))
-        self._lbl_cfg_msg.grid(column=0, row=6, columnspan=6, padx=3, sticky=(W, E))
-        self._lbx_cfg_msg.grid(column=0, row=7, columnspan=3, rowspan=4, padx=3,
+        self._lbl_cfg_msg.grid(column=0, row=8, columnspan=6, padx=3, sticky=(W, E))
+        self._lbx_cfg_msg.grid(column=0, row=9, columnspan=2, rowspan=4, padx=3,
                                pady=3, sticky=(W, E))
-        self._scr_cfg_msg.grid(column=2, row=7, rowspan=4, sticky=(N, S, E))
-        self._chk_ddc.grid(column=3, row=7, padx=0, pady=0, sticky=(W))
-        self._chk_uart1.grid(column=3, row=8, padx=0, pady=0, sticky=(W))
-        self._chk_usb.grid(column=3, row=9, padx=0, pady=0, sticky=(W))
-        self._chk_spi.grid(column=3, row=10, padx=0, pady=0, sticky=(W))
-        self._btn_send_cfg_msg.grid(column=4, row=7, rowspan=4, ipadx=3, ipady=3,
+        self._scr_cfg_msg.grid(column=1, row=9, rowspan=4, sticky=(N, S, E))
+        self._lbl_ddc.grid(column=2, row=9, padx=0, pady=0, sticky=(E))
+        self._spn_ddc.grid(column=3, row=9, padx=0, pady=0, sticky=(W))
+        self._lbl_uart1.grid(column=2, row=10, padx=0, pady=0, sticky=(E))
+        self._spn_uart1.grid(column=3, row=10, padx=0, pady=0, sticky=(W))
+        self._lbl_usb.grid(column=2, row=11, padx=0, pady=0, sticky=(E))
+        self._spn_usb.grid(column=3, row=11, padx=0, pady=0, sticky=(W))
+        self._lbl_spi.grid(column=2, row=12, padx=0, pady=0, sticky=(E))
+        self._spn_spi.grid(column=3, row=12, padx=0, pady=0, sticky=(W))
+        self._btn_send_cfg_msg.grid(column=4, row=9, rowspan=4, ipadx=3, ipady=3,
                                    sticky=(E))
-        self._lbl_send_cfg_msg.grid(column=5, row=7, rowspan=4, ipadx=3,
+        self._lbl_send_cfg_msg.grid(column=5, row=9, rowspan=4, ipadx=3,
                                  ipady=3, sticky=(W))
 
         # *******************************************************
         # PRESET Message Selection
         # *******************************************************
-        ttk.Separator(self._frm_container).grid(column=0, row=14, columnspan=6,
+        ttk.Separator(self._frm_container).grid(column=0, row=16, columnspan=6,
                                                 padx=3, pady=3, sticky=(W, E))
-        self._lbl_presets.grid(column=0, row=15, columnspan=6, padx=3, sticky=(W, E))
-        self._lbx_preset.grid(column=0, row=16, columnspan=4, padx=3, pady=3, sticky=(W, E))
-        self._scr_presetv.grid(column=3, row=16, sticky=(N, S, E))
-        self._scr_preseth.grid(column=0, row=17, columnspan=4, sticky=(W, E))
-        self._btn_send_preset.grid(column=4, row=16, ipadx=3, ipady=3,
+        self._lbl_presets.grid(column=0, row=17, columnspan=6, padx=3, sticky=(W, E))
+        self._lbx_preset.grid(column=0, row=18, columnspan=4, padx=3, pady=3, sticky=(W, E))
+        self._scr_presetv.grid(column=3, row=18, sticky=(N, S, E))
+        self._scr_preseth.grid(column=0, row=19, columnspan=4, sticky=(W, E))
+        self._btn_send_preset.grid(column=4, row=18, ipadx=3, ipady=3,
                                    sticky=(E))
-        self._lbl_send_preset.grid(column=5, row=16, rowspan=2, ipadx=3,
+        self._lbl_send_preset.grid(column=5, row=18, rowspan=2, ipadx=3,
                                  ipady=3, sticky=(W))
 
-        ttk.Separator(self._frm_container).grid(column=0, row=18, columnspan=6,
+        ttk.Separator(self._frm_container).grid(column=0, row=20, columnspan=6,
                                                 padx=3, pady=3, sticky=(W, E))
 
-        self._lbl_status.grid(column=0, row=19, columnspan=4, ipadx=3, ipady=3,
+        self._lbl_status.grid(column=0, row=21, columnspan=4, ipadx=3, ipady=3,
                             sticky=(W, E))
-        self._btn_exit.grid(column=4, row=19, ipadx=3, ipady=3,
+        self._btn_exit.grid(column=4, row=21, ipadx=3, ipady=3,
                             sticky=(E))
 
     def _attach_events(self):
@@ -287,6 +348,14 @@ class UBXConfigDialog():
         available indication of current state.
         '''
 
+        # MON-VER information
+        if cfgtype == 'MON-VER':
+            self._sw_version.set(kwargs.get('swversion', ''))
+            self._hw_version.set(kwargs.get('hwversion', ''))
+            self._fw_version.set(kwargs.get('fwversion', ''))
+            self._protocol.set(kwargs.get('protocol', ''))
+            self._gnss_supported.set(kwargs.get('gnsssupported', ''))
+
         # CFG-PRT command confirmation
         if 'baudrate' in kwargs:
             self._ubx_baudrate.set(str(kwargs['baudrate']))
@@ -300,10 +369,10 @@ class UBXConfigDialog():
         if self._awaiting_cfgmsg:
             if cfgtype == 'CFG-MSG':
                 self.set_status(f"{cfgtype} received", "green")
-                self._ddc_onoff.set(1 if kwargs['ddcrate'] > 0 else 0)
-                self._uart1_onoff.set(1 if kwargs['uart1rate'] > 0 else 0)
-                self._usb_onoff.set(1 if kwargs['usbrate'] > 0 else 0)
-                self._spi_onoff.set(1 if kwargs['spirate'] > 0 else 0)
+                self._ddc_rate.set(kwargs.get('ddcrate', 0))
+                self._uart1_rate.set(kwargs.get('uart1rate', 0))
+                self._usb_rate.set(kwargs.get('usbrate', 0))
+                self._spi_rate.set(kwargs.get('spirate', 0))
                 self._lbl_send_cfg_msg.config(image=self._img_confirmed)
                 self._awaiting_cfgmsg = False
             if cfgtype == 'ACK-NAK':
@@ -340,7 +409,7 @@ class UBXConfigDialog():
         outProtoMask = int.to_bytes(self._ubx_outprot.get(), 2, 'little', signed=False)
         payload = portID + reserved0 + txReady + mode + baudRate + inProtoMask \
                   +outProtoMask + reserved4 + reserved5
-        msg = UBXMessage('CFG', 'CFG-PRT', payload, SET)
+        msg = UBXMessage('CFG', 'CFG-PRT', SET, payload=payload)
         self.__app.serial_handler.serial_write(msg.serialize())
 
         self._do_poll_prt()  # poll for confirmation
@@ -368,14 +437,16 @@ class UBXConfigDialog():
         '''
 
         msg = UBXMessage.key_from_val(UBX_CONFIG_MESSAGES, self._cfg_msg_command)
-        rateDDC = b'\x00' if self._ddc_onoff.get() == 0 else b'\x01'
-        rateUART1 = b'\x00' if self._uart1_onoff.get() == 0 else b'\x01'
-        rateUART2 = b'\x00'
-        rateUSB = b'\x00' if self._usb_onoff.get() == 0 else b'\x01'
-        rateSPI = b'\x00' if self._spi_onoff.get() == 0 else b'\x01'
-        reserved = b'\x00'
-        msg_payload = msg + rateDDC + rateUART1 + rateUART2 + rateUSB + rateSPI + reserved
-        data = UBXMessage('CFG', 'CFG-MSG', SET, payload=msg_payload)
+        msgClass = int.from_bytes(msg[0:1], 'little', signed=False)
+        msgID = int.from_bytes(msg[1:2], 'little', signed=False)
+        rateDDC = int(self._ddc_rate.get())
+        rateUART1 = int(self._uart1_rate.get())
+        rateUSB = int(self._usb_rate.get())
+        rateSPI = int(self._spi_rate.get())
+        data = UBXMessage('CFG', 'CFG-MSG', SET,
+                          msgClass=msgClass, msgID=msgID,
+                          rateDDC=rateDDC, rateUART1=rateUART1,
+                          rateUSB=rateUSB, rateSPI=rateSPI)
         self.__app.serial_handler.serial_write(data.serialize())
         data = UBXMessage('CFG', 'CFG-MSG', POLL, payload=msg)  # poll for a response
         self.__app.serial_handler.serial_write(data.serialize())
@@ -407,35 +478,37 @@ class UBXConfigDialog():
             elif self._preset_command == PSTMINNMEAON:
                 self._do_set_minnmea()
             elif self._preset_command == PSTALLNMEAON:
-                self._do_set_allnmea(True)
+                self._do_set_allnmea(1)
             elif self._preset_command == PSTALLNMEAOFF:
-                self._do_set_allnmea(False)
+                self._do_set_allnmea(0)
             elif self._preset_command == PSTMINUBXON:
                 self._do_set_minNAV()
             elif self._preset_command == PSTALLUBXON:
-                self._do_set_allNAV(True)
+                self._do_set_allNAV(1)
             elif self._preset_command == PSTALLUBXOFF:
-                self._do_set_allNAV(False)
+                self._do_set_allNAV(0)
             elif self._preset_command == PSTALLINFON:
                 self._do_set_inf(True)
             elif self._preset_command == PSTALLINFOFF:
                 self._do_set_inf(False)
             elif self._preset_command == PSTALLLOGON:
-                self._do_set_log(True)
+                self._do_set_log(4)
             elif self._preset_command == PSTALLLOGOFF:
-                self._do_set_log(False)
+                self._do_set_log(0)
             elif self._preset_command == PSTALLMONON:
-                self._do_set_mon(True)
+                self._do_set_mon(4)
             elif self._preset_command == PSTALLMONOFF:
-                self._do_set_mon(False)
+                self._do_set_mon(0)
             elif self._preset_command == PSTALLRXMON:
-                self._do_set_rxm(True)
+                self._do_set_rxm(4)
             elif self._preset_command == PSTALLRXMOFF:
-                self._do_set_rxm(False)
+                self._do_set_rxm(0)
             elif self._preset_command == PSTPOLLPORT:
                 self._do_poll_prt()
             elif self._preset_command == PSTPOLLINFO:
                 self._do_poll_inf()
+            elif self._preset_command == PSTPOLLALL:
+                self._do_poll_all()
             else:
                 self._do_user_defined(self._preset_command)
 
@@ -450,6 +523,17 @@ class UBXConfigDialog():
             self.set_status(f"Error {err}", "red")
             self._lbl_send_preset.config(image=self._img_warn)
 
+    def _do_poll_all(self):
+        '''
+        Poll INF message configuration
+        '''
+
+        for msgtype in UBX_PAYLOADS_POLL:
+            if msgtype[0:3] == 'CFG' and \
+                msgtype not in ('CFG-INF', 'CFG-MSG', 'CFG-PRT-IO', 'CFG-TP5-TPX'):
+                msg = UBXMessage('CFG', msgtype, POLL)
+                self.__app.serial_handler.serial_write(msg.serialize())
+
     def _do_poll_inf(self):
         '''
         Poll INF message configuration
@@ -491,32 +575,32 @@ class UBXConfigDialog():
             self.__app.serial_handler.serial_write(msg.serialize())
             self._do_poll_inf()  # poll results
 
-    def _do_set_log(self, onoff):
+    def _do_set_log(self, msgrate):
         '''
         Turn on all device logging messages LOG
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\x21':
-                self._do_cfgmsg(msgtype, onoff)
+                self._do_cfgmsg(msgtype, msgrate)
 
-    def _do_set_mon(self, onoff):
+    def _do_set_mon(self, msgrate):
         '''
         Turn on all device monitoring messages MON
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\x0A':
-                self._do_cfgmsg(msgtype, onoff)
+                self._do_cfgmsg(msgtype, msgrate)
 
-    def _do_set_rxm(self, onoff):
+    def _do_set_rxm(self, msgrate):
         '''
         Turn on all device receiver management messages RXM
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\x02':
-                self._do_cfgmsg(msgtype, onoff)
+                self._do_cfgmsg(msgtype, msgrate)
 
     def _do_set_minnmea(self):
         '''
@@ -525,12 +609,14 @@ class UBXConfigDialog():
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\xf0':  # standard NMEA
-                if msgtype in (b'\xf0\x00', b'\xf0\x02', b'\xf0\x03'):  # GGA, GSA, GSV
-                    self._do_cfgmsg(msgtype, True)
+                if msgtype in (b'\xf0\x00', b'\xf0\x02'):  # GGA, GSA
+                    self._do_cfgmsg(msgtype, 1)
+                elif msgtype == b'\xf0\x03':  # GSV
+                    self._do_cfgmsg(msgtype, 4)
                 else:
-                    self._do_cfgmsg(msgtype, False)
+                    self._do_cfgmsg(msgtype, 0)
             if msgtype[0:1] == b'\xf1':  # proprietary NMEA
-                self._do_cfgmsg(msgtype, False)
+                self._do_cfgmsg(msgtype, 0)
 
     def _do_set_minNAV(self):
         '''
@@ -539,40 +625,45 @@ class UBXConfigDialog():
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\x01':  # UBX-NAV
-                if msgtype in (b'\x01\x07', b'\x01\x30'):  # NAV-PVT, NAV-SVINFO
-                    self._do_cfgmsg(msgtype, True)
+                if msgtype == b'\x01\x07':  # NAV-PVT
+                    self._do_cfgmsg(msgtype, 1)
+                elif msgtype == b'\x01\x30':  # NAV-SVINFO
+                    self._do_cfgmsg(msgtype, 4)
                 else:
-                    self._do_cfgmsg(msgtype, False)
+                    self._do_cfgmsg(msgtype, 0)
 
-    def _do_set_allnmea(self, onoff):
+    def _do_set_allnmea(self, msgrate):
         '''
         Turn on all NMEA messages
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] not in (b'\x0a', b'\x01'):
-                self._do_cfgmsg(msgtype, onoff)
+                self._do_cfgmsg(msgtype, msgrate)
 
-    def _do_set_allNAV(self, onoff):
+    def _do_set_allNAV(self, msgrate):
         '''
         Turn on all UBX-NAV messages
         '''
 
         for msgtype in UBX_CONFIG_MESSAGES:
             if msgtype[0:1] == b'\x01':
-                self._do_cfgmsg(msgtype, onoff)
+                self._do_cfgmsg(msgtype, msgrate)
 
-    def _do_cfgmsg(self, msgtype, onoff):
+    def _do_cfgmsg(self, msgtype, msgrate):
         '''
-        Send CFG-MSG for specified message type
+        Set rate for specified message type via CFG-MSG
+
+        NB A rate of n means 'for every nth position solution',
+        so values > 1 mean the message is sent less often.
         '''
 
-        if onoff:
-            rate = CFG_MSG_ON
-        else:
-            rate = CFG_MSG_OFF
-        payload = msgtype + rate
-        msg = UBXMessage('CFG', 'CFG-MSG', SET, payload=payload)
+        msgClass = int.from_bytes(msgtype[0:1], 'little', signed=False)
+        msgID = int.from_bytes(msgtype[1:2], 'little', signed=False)
+        msg = UBXMessage('CFG', 'CFG-MSG', SET,
+                         msgClass=msgClass, msgID=msgID,
+                         rateDDC=msgrate, rateUART1=msgrate,
+                         rateUSB=msgrate, rateSPI=msgrate)
         self.__app.serial_handler.serial_write(msg.serialize())
 
     def _do_factory_reset(self) -> bool:
@@ -583,11 +674,10 @@ class UBXConfigDialog():
 
         if messagebox.askokcancel(DLGRESET, DLGRESETCONFIRM):
             clearMask = b'\x1f\x1f\x00\x00'
-            saveMask = b'\x00\x00\x00\x00'
             loadMask = b'\x1f\x1f\x00\x00'
-            devicerMask = b'\x01'  # battery backed RAM
-            payload = clearMask + saveMask + loadMask + devicerMask
-            msg = UBXMessage('CFG', 'CFG-CFG', SET, payload=payload)
+            deviceMask = b'\x07'  # target RAM, Flash and EEPROM
+            msg = UBXMessage('CFG', 'CFG-CFG', SET, clearMask=clearMask,
+                             loadMask=loadMask, deviceMask=deviceMask)
             self.__app.serial_handler.serial_write(msg.serialize())
             return True
 
@@ -595,17 +685,15 @@ class UBXConfigDialog():
 
     def _do_save_config(self) -> bool:
         '''
-        Save current configuration to battery-backed RAM
+        Save current configuration to persistent storage
         but display confirmation message box first.
         '''
 
         if messagebox.askokcancel(DLGSAVE, DLGSAVECONFIRM):
-            clearMask = b'\x00\x00\x00\x00'
             saveMask = b'\x1f\x1f\x00\x00'
-            loadMask = b'\x00\x00\x00\x00'
-            devicerMask = b'\x01'  # battery backed RAM
-            payload = clearMask + saveMask + loadMask + devicerMask
-            msg = UBXMessage('CFG', 'CFG-CFG', SET, payload=payload)
+            deviceMask = b'\x07'  # target RAM, Flash and EEPROM
+            msg = UBXMessage('CFG', 'CFG-CFG', SET, saveMask=saveMask,
+                             deviceMask=deviceMask)
             self.__app.serial_handler.serial_write(msg.serialize())
             return True
 
